package com.example.wc24h;

import android.app.Dialog;
import android.content.pm.ActivityInfo;
import android.view.Gravity;
import android.view.View;
import android.widget.Button;
import android.widget.LinearLayout;

import com.ravensoft.daniel.joystick.JoystickView;

class SnakeDialog extends Dialog implements JoystickView.JoystickListener
{
    private MainActivity        mContext        = null;
    private byte[]                      msg             = new byte[34];
    private View                        leftButtons;
    private View                        rightButtons;
    private LinearLayout                joystickContainer;
    private final static int            angleThreshold = 44;
    private final static int            distThreshold = 10;
    private final static int            NONE  = 0;
    private final static int            LEFT  = 1;
    private final static int            RIGHT = 2;
    private final static int            UP    = 3;
    private final static int            DOWN  = 4;
    private int                         direction = NONE;
    private final double                RAD = 57.2957795;

    SnakeDialog(MainActivity a)
    {
        super(a, R.style.dialog_custom_style);
        mContext = a;
        setContentView(R.layout.snake_dialog);

        JoystickView joystick = findViewById(R.id.joystick);
        joystick.setListener(this);

        joystickContainer = findViewById(R.id.joystick_container);

        leftButtons = findViewById(R.id.buttons_left);
        rightButtons = findViewById(R.id.buttons_right);

        rightButtons.setVisibility(View.GONE);

        msg[0] = 'g';
        msg[1] = 'S';
        msg[3] = '\0';

        Button btn = (Button)findViewById(R.id.start_btn_left);
        btn.setOnClickListener(new View.OnClickListener() {
            @Override
            public void onClick(View view) {
                msg[2] = 's';
                mContext.SendMessage(msg, 4);
            }
        });
        btn = (Button)findViewById(R.id.start_btn_right);
        btn.setOnClickListener(new View.OnClickListener() {
            @Override
            public void onClick(View view) {
                msg[2] = 's';
                mContext.SendMessage(msg, 4);
            }
        });
        btn = (Button)findViewById(R.id.end_btn_left);
        btn.setOnClickListener(new View.OnClickListener() {
            @Override
            public void onClick(View view) {
                msg[2] = 'q';
                mContext.SendMessage(msg, 4);
                SnakeDialog.this.dismiss();
            }
        });
        btn = (Button)findViewById(R.id.end_btn_right);
        btn.setOnClickListener(new View.OnClickListener() {
            @Override
            public void onClick(View view) {
                msg[2] = 'q';
                mContext.SendMessage(msg, 4);
                SnakeDialog.this.dismiss();
            }
        });

        // left button
        final Button rl = findViewById(R.id.left_hand_btn);
        rl.setOnClickListener(new View.OnClickListener() {
            @Override
            public void onClick(View view) {
                leftButtons.setVisibility(View.GONE);
                rightButtons.setVisibility(View.VISIBLE);
                LinearLayout.LayoutParams params = new LinearLayout.LayoutParams(LinearLayout.LayoutParams.WRAP_CONTENT, LinearLayout.LayoutParams.FILL_PARENT);
                params.gravity = Gravity.LEFT;
                joystickContainer.setLayoutParams(params);
            }
        });

        // right radio buttons
        final Button rb = findViewById(R.id.right_hand_btn);
        rb.setOnClickListener(new View.OnClickListener() {
            @Override
            public void onClick(View view) {
                leftButtons.setVisibility(View.VISIBLE);
                rightButtons.setVisibility(View.GONE);
                LinearLayout.LayoutParams params = new LinearLayout.LayoutParams(LinearLayout.LayoutParams.WRAP_CONTENT, LinearLayout.LayoutParams.FILL_PARENT);
                params.gravity = Gravity.RIGHT;
                joystickContainer.setLayoutParams(params);
            }
        });
    }

    @Override
    protected void onStart()
    {
        super.onStart();
        mContext.setRequestedOrientation(ActivityInfo.SCREEN_ORIENTATION_LANDSCAPE);
    }

    @Override
    protected void onStop()
    {
        super.onStop();
        mContext.setRequestedOrientation(ActivityInfo.SCREEN_ORIENTATION_PORTRAIT);
    }

    //String str = "-";
    @Override
    public void onJoystickMoved(float xPercent, float yPercent, int id) {
        int nextDirection = NONE;
        int angle = getAngle(xPercent, yPercent) + 180;
        int dist = getPower(xPercent, yPercent);

        if (angle > (180 - angleThreshold) && angle < (180 + angleThreshold)) {
            nextDirection = UP;
        } else if (angle < (angleThreshold) || angle > (360 - angleThreshold)) {
            nextDirection = DOWN;
        } else if (angle > (90 - angleThreshold) && angle < (90 + angleThreshold)) {
            nextDirection = LEFT;
        } else if (angle > (270 - angleThreshold) && angle < (270 + angleThreshold)) {
            nextDirection = RIGHT;
        }
        if (dist > distThreshold && direction != nextDirection) {
            //str = "-";
            switch(nextDirection) {
                case LEFT:
                    msg[2] = 'l';
                    //str = "L";
                    mContext.SendMessage(msg, 4);
                    break;
                case RIGHT:
                    msg[2] = 'r';
                    //str = "R";
                    mContext.SendMessage(msg, 4);
                    break;
                case UP:
                    msg[2] = 'u';
                    //str = "U";
                    mContext.SendMessage(msg, 4);
                    break;
                case DOWN:
                    msg[2] = 'd';
                    //str = "D";
                    mContext.SendMessage(msg, 4);
                    break;
            }
            //System.out.println("Direction: " + nextDirection + "  angle= " + angle);
            direction = nextDirection;
        }
        //System.out.println("Direction: " + str + "  Angle= " + angle + "  Distance= " + dist);
        //System.out.println("Direction: " + nextDirection + "<>" + direction + "  Angle= " + angle + "  Distance= " + dist);
        //System.out.println("X percent: " + xPercent + " Y percent: " + yPercent);
    }

    private int getAngle(float xPosition, float yPosition) {
        float centerX = 0;
        float centerY = 0;
        if (xPosition > centerX) {
            if (yPosition < centerY) {
                return (int) (Math.atan((yPosition - centerY)
                        / (xPosition - centerX))
                        * RAD + 90);
            } else if (yPosition > centerY) {
                return (int) (Math.atan((yPosition - centerY)
                        / (xPosition - centerX)) * RAD) + 90;
            } else {
                return 90;
            }
        } else if (xPosition < centerX) {
            if (yPosition < centerY) {
                return (int) (Math.atan((yPosition - centerY)
                        / (xPosition - centerX))
                        * RAD - 90);
            } else if (yPosition > centerY) {
                return (int) (Math.atan((yPosition - centerY)
                        / (xPosition - centerX)) * RAD) - 90;
            } else {
                return -90;
            }
        } else {
              return 0;
//            if (yPosition <= centerY) {
//                return 0;
//            } else {
//                if (lastAngle < 0) {
//                    return lastAngle = -180;
//                } else {
//                    return lastAngle = 180;
//                }
//            }
        }
    }

    private int getPower(float xPosition, float yPosition) {
        float centerX = 0;
        float centerY = 0;
        return (int) (100 * Math.sqrt((xPosition - centerX)
                * (xPosition - centerX) + (yPosition - centerY)
                * (yPosition - centerY))/* / joystickRadius*/);
    }
}